@@ -24,8 +24,8 @@
 
 #include "glocaldirectorymonitor.h"
 #include "gunixmounts.h"
-#include "gdirectorymonitor.h"
 #include "giomodule-priv.h"
+#include "glibintl.h"
 
 #include <string.h>
 
@@ -37,15 +37,16 @@ enum
   PROP_DIRNAME
 };
 
-static gboolean g_local_directory_monitor_cancel (GDirectoryMonitor* monitor);
-static void mounts_changed (GUnixMountMonitor *mount_monitor, gpointer user_data);
+static gboolean g_local_directory_monitor_cancel (GFileMonitor      *monitor);
+static void     mounts_changed                   (GUnixMountMonitor *mount_monitor, 
+                                                  gpointer           user_data);
 
-G_DEFINE_ABSTRACT_TYPE (GLocalDirectoryMonitor, g_local_directory_monitor, G_TYPE_DIRECTORY_MONITOR)
+G_DEFINE_ABSTRACT_TYPE (GLocalDirectoryMonitor, g_local_directory_monitor, G_TYPE_FILE_MONITOR)
 
 static void
-g_local_directory_monitor_finalize (GObject* object)
+g_local_directory_monitor_finalize (GObject *object)
 {
-  GLocalDirectoryMonitor* local_monitor;
+  GLocalDirectoryMonitor *local_monitor;
   local_monitor = G_LOCAL_DIRECTORY_MONITOR (object);
 
   g_free (local_monitor->dirname);
@@ -139,23 +140,29 @@ static void
 g_local_directory_monitor_class_init (GLocalDirectoryMonitorClass* klass)
 {
   GObjectClass* gobject_class = G_OBJECT_CLASS (klass);
-  GDirectoryMonitorClass *dir_monitor_class = G_DIRECTORY_MONITOR_CLASS (klass);
+  GFileMonitorClass *file_monitor_class = G_FILE_MONITOR_CLASS (klass);
   
   gobject_class->finalize = g_local_directory_monitor_finalize;
   gobject_class->set_property = g_local_directory_monitor_set_property;
   gobject_class->constructor = g_local_directory_monitor_constructor;
 
-  dir_monitor_class->cancel = g_local_directory_monitor_cancel;
+  file_monitor_class->cancel = g_local_directory_monitor_cancel;
 
-  g_object_class_install_property (gobject_class, PROP_DIRNAME,
-    g_param_spec_string ("dirname", "Directory name", "Directory to monitor",
-        NULL, G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
+  g_object_class_install_property (gobject_class, 
+                                   PROP_DIRNAME,
+                                   g_param_spec_string ("dirname", 
+                                                        P_("Directory name"), 
+                                                        P_("Directory to monitor"),
+                                                        NULL, 
+                                                        G_PARAM_CONSTRUCT_ONLY|
+                                                        G_PARAM_WRITABLE|
+                                                        G_PARAM_STATIC_NAME|G_PARAM_STATIC_NICK|G_PARAM_STATIC_BLURB));
 
   klass->mount_notify = FALSE;
 }
 
 static void
-g_local_directory_monitor_init (GLocalDirectoryMonitor* local_monitor)
+g_local_directory_monitor_init (GLocalDirectoryMonitor *local_monitor)
 {
 }
 
@@ -186,63 +193,38 @@ mounts_changed (GUnixMountMonitor *mount_monitor,
       if (local_monitor->was_mounted && !is_mounted)
         {
           file = g_file_new_for_path (local_monitor->dirname);
-          g_directory_monitor_emit_event (G_DIRECTORY_MONITOR (local_monitor),
-                                          file, NULL,
-                                          G_FILE_MONITOR_EVENT_UNMOUNTED);
+          g_file_monitor_emit_event (G_FILE_MONITOR (local_monitor),
+				     file, NULL,
+				     G_FILE_MONITOR_EVENT_UNMOUNTED);
           g_object_unref (file);
         }
       local_monitor->was_mounted = is_mounted;
     }
 }
 
-static gint
-_compare_monitor_type_by_prio (gconstpointer _a,
-			       gconstpointer _b,
-			       gpointer      user_data)
-{
-  const GType *a = _a, *b = _b;
-  int prio_a, prio_b;
-  gint ret;
-  GQuark private_q;
-
-  private_q = g_quark_from_static_string ("gio-prio");
-
-  prio_a = GPOINTER_TO_INT (g_type_get_qdata (*a, private_q));
-  prio_b = GPOINTER_TO_INT (g_type_get_qdata (*b, private_q));
-
-  ret = prio_b - prio_a;
-
-  return ret;
-}
-
 static gpointer
 get_default_local_directory_monitor (gpointer data)
 {
-  GType *monitor_impls;
-  guint n_monitor_impls;
-  gint i;
   GLocalDirectoryMonitorClass *chosen_class;
   GLocalDirectoryMonitorClass **ret = data;
+  GIOExtensionPoint *ep;
+  GList *extensions, *l;
 
   _g_io_modules_ensure_loaded ();
+
+  ep = g_io_extension_point_lookup (G_LOCAL_DIRECTORY_MONITOR_EXTENSION_POINT_NAME);
+
+  extensions = g_io_extension_point_get_extensions (ep);
   
-  monitor_impls = g_type_children (G_TYPE_LOCAL_DIRECTORY_MONITOR,
-                                   &n_monitor_impls);
-
-  g_qsort_with_data (monitor_impls,
-                     n_monitor_impls,
-                     sizeof (GType),
-                     _compare_monitor_type_by_prio,
-                     NULL);
-
   chosen_class = NULL;
-  for (i = 0; i < n_monitor_impls; i++)
-    {    
+  for (l = extensions; l != NULL; l = l->next)
+    {
+      GIOExtension *extension = l->data;
       GLocalDirectoryMonitorClass *klass;
       
-      klass = G_LOCAL_DIRECTORY_MONITOR_CLASS (g_type_class_ref (monitor_impls[i]));
+      klass = G_LOCAL_DIRECTORY_MONITOR_CLASS (g_io_extension_ref_class (extension));
       
-      if (klass->is_supported())
+      if (klass->is_supported ())
 	{
 	  chosen_class = klass;
 	  break;
@@ -250,8 +232,6 @@ get_default_local_directory_monitor (gpointer data)
       else
 	g_type_class_unref (klass);
     }
-
-  g_free (monitor_impls);
   
   if (chosen_class)
     {
@@ -263,19 +243,20 @@ get_default_local_directory_monitor (gpointer data)
 }
 
 /**
- * g_local_directory_monitor_new:
+ * _g_local_directory_monitor_new:
  * @dirname: filename of the directory to monitor.
  * @flags: #GFileMonitorFlags.
  * 
- * Returns: new #GDirectoryMonitor for the given @dirname.
+ * Returns: new #GFileMonitor for the given @dirname.
  **/
-GDirectoryMonitor*
-_g_local_directory_monitor_new (const char*       dirname,
-				GFileMonitorFlags flags)
+GFileMonitor*
+_g_local_directory_monitor_new (const char         *dirname,
+				GFileMonitorFlags   flags,
+				GError            **error)
 {
   static GOnce once_init = G_ONCE_INIT;
   GTypeClass *type_class;
-  GDirectoryMonitor *monitor;
+  GFileMonitor *monitor;
   GType type;
 
   type_class = NULL;
@@ -284,7 +265,9 @@ _g_local_directory_monitor_new (const char*       dirname,
 
   monitor = NULL;
   if (type != G_TYPE_INVALID)
-    monitor = G_DIRECTORY_MONITOR (g_object_new (type, "dirname", dirname, NULL));
+    monitor = G_FILE_MONITOR (g_object_new (type, "dirname", dirname, NULL));
+  else
+    g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED, _("Unable to find default local directory monitor type"));
 
   /* This is non-null on first pass here. Unref the class now.
    * This is to avoid unloading the module and then loading it
@@ -299,7 +282,7 @@ _g_local_directory_monitor_new (const char*       dirname,
 }
 
 static gboolean
-g_local_directory_monitor_cancel (GDirectoryMonitor* monitor)
+g_local_directory_monitor_cancel (GFileMonitor *monitor)
 {
   GLocalDirectoryMonitor *local_monitor = G_LOCAL_DIRECTORY_MONITOR (monitor);
 
