@@ -46,6 +46,10 @@
 #include <sys/mount.h>
 #endif
 
+#ifndef O_BINARY
+#define O_BINARY 0
+#endif
+
 #if defined(HAVE_STATFS) && defined(HAVE_STATVFS)
 /* Some systems have both statfs and statvfs, pick the
    most "native" for these */
@@ -80,6 +84,7 @@
 #include "glibintl.h"
 
 #ifdef G_OS_WIN32
+#define _WIN32_WINNT 0x0500
 #include <windows.h>
 #include <io.h>
 #include <direct.h>
@@ -143,6 +148,7 @@ g_local_file_class_init (GLocalFileClass *klass)
   g_file_attribute_info_list_add (list,
 				  G_FILE_ATTRIBUTE_UNIX_MODE,
 				  G_FILE_ATTRIBUTE_TYPE_UINT32,
+				  G_FILE_ATTRIBUTE_INFO_COPY_WITH_FILE |
 				  G_FILE_ATTRIBUTE_INFO_COPY_WHEN_MOVED);
   
 #ifdef HAVE_CHOWN
@@ -514,8 +520,8 @@ match_prefix (const char *path,
 }
 
 static gboolean
-g_local_file_contains_file (GFile *parent,
-			    GFile *descendant)
+g_local_file_prefix_matches (GFile *parent,
+			     GFile *descendant)
 {
   GLocalFile *parent_local = G_LOCAL_FILE (parent);
   GLocalFile *descendant_local = G_LOCAL_FILE (descendant);
@@ -789,6 +795,111 @@ get_mount_info (GFileInfo             *fs_info,
 
 #endif
 
+#ifdef G_OS_WIN32
+
+static gboolean
+is_xp_or_later (void)
+{
+  static int result = -1;
+
+  if (result == -1)
+    {
+      OSVERSIONINFOEX ver_info = {0};
+      DWORDLONG cond_mask = 0;
+      int op = VER_GREATER_EQUAL;
+
+      ver_info.dwOSVersionInfoSize = sizeof ver_info;
+      ver_info.dwMajorVersion = 5;
+      ver_info.dwMinorVersion = 1;
+
+      VER_SET_CONDITION (cond_mask, VER_MAJORVERSION, op);
+      VER_SET_CONDITION (cond_mask, VER_MINORVERSION, op);
+
+      result = VerifyVersionInfo (&ver_info,
+				  VER_MAJORVERSION | VER_MINORVERSION, 
+				  cond_mask) != 0;
+    }
+
+  return result;
+}
+
+static wchar_t *
+get_volume_for_path (const char *path)
+{
+  long len;
+  wchar_t *wpath;
+  wchar_t *result;
+
+  wpath = g_utf8_to_utf16 (path, -1, NULL, &len, NULL);
+  result = g_new (wchar_t, len + 2);
+
+  if (!GetVolumePathNameW (wpath, result, len + 2))
+    {
+      char *msg = g_win32_error_message (GetLastError ());
+      g_critical ("GetVolumePathName failed: %s", msg);
+      g_free (msg);
+      g_free (result);
+      g_free (wpath);
+      return NULL;
+    }
+
+  len = wcslen (result);
+  if (len > 0 && result[len-1] != L'\\')
+    {
+      result = g_renew (wchar_t, result, len + 2);
+      result[len] = L'\\';
+      result[len + 1] = 0;
+    }
+
+  g_free (wpath);
+  return result;
+}
+
+static char *
+find_mountpoint_for (const char *file, dev_t dev)
+{
+  wchar_t *wpath;
+  char *utf8_path;
+
+  wpath = get_volume_for_path (file);
+  if (!wpath)
+    return NULL;
+
+  utf8_path = g_utf16_to_utf8 (wpath, -1, NULL, NULL, NULL);
+
+  g_free (wpath);
+  return utf8_path;
+}
+
+static void
+get_filesystem_readonly (GFileInfo  *info,
+			 const char *path)
+{
+  wchar_t *rootdir;
+
+  rootdir = get_volume_for_path (path);
+
+  if (rootdir)
+    {
+      if (is_xp_or_later ())
+        {
+          DWORD flags;
+          if (GetVolumeInformationW (rootdir, NULL, 0, NULL, NULL, &flags, NULL, 0))
+	    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY,
+					       (flags & FILE_READ_ONLY_VOLUME) != 0);
+        }
+      else
+        {
+          if (GetDriveTypeW (rootdir) == DRIVE_CDROM)
+	    g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_FILESYSTEM_READONLY, TRUE);
+        }
+    }
+
+  g_free (rootdir);
+}
+
+#endif /* G_OS_WIN32 */
+
 static GFileInfo *
 g_local_file_query_filesystem_info (GFile         *file,
 				    const char    *attributes,
@@ -840,10 +951,12 @@ g_local_file_query_filesystem_info (GFile         *file,
 
   if (statfs_result == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error getting filesystem info: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return NULL;
     }
 
@@ -895,7 +1008,7 @@ g_local_file_query_filesystem_info (GFile         *file,
 					G_FILE_ATTRIBUTE_FILESYSTEM_READONLY))
     {
 #ifdef G_OS_WIN32
-      /* need to implement with *unix_mount* */
+      get_filesystem_readonly (info, local->filename);
 #else
       get_mount_info (info, local->filename, attribute_matcher);
 #endif
@@ -918,18 +1031,22 @@ g_local_file_find_enclosing_mount (GFile         *file,
 
   if (g_lstat (local->filename, &buf) != 0)
     {
-      g_set_error (error, G_IO_ERROR,
-		   G_IO_ERROR_NOT_FOUND,
-		   _("Containing mount does not exist"));
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+		      /* Translators: This is an error message when trying to
+		       * find the enclosing (user visible) mount of a file, but
+		       * none exists. */
+		      _("Containing mount does not exist"));
       return NULL;
     }
 
   mountpoint = find_mountpoint_for (local->filename, buf.st_dev);
   if (mountpoint == NULL)
     {
-      g_set_error (error, G_IO_ERROR,
-		   G_IO_ERROR_NOT_FOUND,
-		   _("Containing mount does not exist"));
+      g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+		      /* Translators: This is an error message when trying to
+		       * find the enclosing (user visible) mount of a file, but
+		       * none exists. */
+		      _("Containing mount does not exist"));
       return NULL;
     }
 
@@ -938,9 +1055,11 @@ g_local_file_find_enclosing_mount (GFile         *file,
   if (mount)
     return mount;
 
-  g_set_error (error, G_IO_ERROR,
-	       G_IO_ERROR_NOT_FOUND,
-	       _("Containing mount does not exist"));
+  g_set_error (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
+		  /* Translators: This is an error message when trying to find
+		   * the enclosing (user visible) mount of a file, but none
+		   * exists. */
+		  _("Containing mount does not exist"));
   return NULL;
 }
 
@@ -982,7 +1101,7 @@ g_local_file_set_display_name (GFile         *file,
       return NULL;
     }
 
-  if (rename (local->filename, new_local->filename) == -1)
+  if (g_rename (local->filename, new_local->filename) == -1)
     {
       errsv = errno;
 
@@ -1107,13 +1226,15 @@ g_local_file_read (GFile         *file,
   int fd;
   struct stat buf;
   
-  fd = g_open (local->filename, O_RDONLY, 0);
+  fd = g_open (local->filename, O_RDONLY|O_BINARY, 0);
   if (fd == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error opening file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return NULL;
     }
 
@@ -1172,10 +1293,12 @@ g_local_file_delete (GFile         *file,
   
   if (g_remove (local->filename) == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error removing file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   
@@ -1305,6 +1428,8 @@ expand_all_symlinks (const char *path)
   return res;
 }
 
+#ifndef G_OS_WIN32
+
 static char *
 find_mountpoint_for (const char *file, 
                      dev_t       dev)
@@ -1332,8 +1457,6 @@ find_mountpoint_for (const char *file,
     }
 }
 
-#ifndef G_OS_WIN32
-
 static char *
 find_topdir_for (const char *file)
 {
@@ -1347,8 +1470,6 @@ find_topdir_for (const char *file)
   return find_mountpoint_for (dir, dir_dev);
 }
 
-#endif
-
 static char *
 get_unique_filename (const char *basename, 
                      int         id)
@@ -1457,17 +1578,17 @@ g_local_file_trash (GFile         *file,
   gboolean is_homedir_trash;
   char delete_time[32];
   int fd;
-#ifndef G_OS_WIN32
   struct stat trash_stat, global_stat;
   char *dirname, *globaldir;
-#endif
   
   if (g_lstat (local->filename, &file_stat) != 0)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error trashing file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
     
@@ -1484,14 +1605,13 @@ g_local_file_trash (GFile         *file,
       if (g_mkdir_with_parents (trashdir, 0700) < 0)
 	{
           char *display_name;
-          int err;
+          int errsv = errno;
 
-          err = errno;
           display_name = g_filename_display_name (trashdir);
           g_set_error (error, G_IO_ERROR,
-                       g_io_error_from_errno (err),
+                       g_io_error_from_errno (errsv),
                        _("Unable to create trash dir %s: %s"),
-                       display_name, g_strerror (err));
+                       display_name, g_strerror (errsv));
           g_free (display_name);
           g_free (trashdir);
           return FALSE;
@@ -1500,9 +1620,6 @@ g_local_file_trash (GFile         *file,
     }
   else
     {
-#ifdef G_OS_WIN32
-      g_warning ("Recycle bin not implemented");
-#else
       uid_t uid;
       char uid_str[32];
 
@@ -1546,28 +1663,47 @@ g_local_file_trash (GFile         *file,
 
       if (trashdir == NULL)
 	{
+	  gboolean tried_create;
+	  
 	  /* No global trash dir, or it failed the tests, fall back to $topdir/.Trash-$uid */
 	  dirname = g_strdup_printf (".Trash-%s", uid_str);
 	  trashdir = g_build_filename (topdir, dirname, NULL);
 	  g_free (dirname);
-	  
+
+	  tried_create = FALSE;
+
+	retry:
 	  if (g_lstat (trashdir, &trash_stat) == 0)
 	    {
 	      if (!S_ISDIR (trash_stat.st_mode) ||
 		  trash_stat.st_uid != uid)
 		{
+		  /* Remove the failed directory */
+		  if (tried_create)
+		    g_remove (trashdir);
+		  
 		  /* Not a directory or not owned by user, ignore */
 		  g_free (trashdir);
 		  trashdir = NULL;
 		}
 	    }
-	  else if (g_mkdir (trashdir, 0700) == -1)
+	  else
 	    {
-	      g_free (trashdir);
-	      trashdir = NULL;
+	      if (!tried_create &&
+		  g_mkdir (trashdir, 0700) != -1)
+		{
+		  /* Ensure that the created dir has the right uid etc.
+		     This might fail on e.g. a FAT dir */
+		  tried_create = TRUE;
+		  goto retry;
+		}
+	      else
+		{
+		  g_free (trashdir);
+		  trashdir = NULL;
+		}
 	    }
 	}
-#endif
 
       if (trashdir == NULL)
 	{
@@ -1619,15 +1755,17 @@ g_local_file_trash (GFile         *file,
 
   if (fd == -1)
     {
+      int errsv = errno;
+
       g_free (filesdir);
       g_free (topdir);
       g_free (trashname);
       g_free (infofile);
       
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Unable to create trashing info file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
 
@@ -1642,15 +1780,17 @@ g_local_file_trash (GFile         *file,
 
   if (g_rename (local->filename, trashfile) == -1)
     {
+      int errsv = errno;
+
       g_free (topdir);
       g_free (trashname);
       g_free (infofile);
       g_free (trashfile);
       
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Unable to trash file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
 
@@ -1668,13 +1808,6 @@ g_local_file_trash (GFile         *file,
   g_free (original_name);
   g_free (topdir);
   
-#ifdef G_OS_WIN32
-  {
-    GTimeVal now;
-    g_get_current_time (&now);
-    strncpy (delete_time, g_time_val_to_iso8601 (&now), sizeof (delete_time));
-  }
-#else
   {
     time_t t;
     struct tm now;
@@ -1683,7 +1816,6 @@ g_local_file_trash (GFile         *file,
     delete_time[0] = 0;
     strftime(delete_time, sizeof (delete_time), "%Y-%m-%dT%H:%M:%S", &now);
   }
-#endif
 
   data = g_strdup_printf ("[Trash Info]\nPath=%s\nDeletionDate=%s\n",
 			  original_name_escaped, delete_time);
@@ -1697,6 +1829,49 @@ g_local_file_trash (GFile         *file,
   
   return TRUE;
 }
+#else /* G_OS_WIN32 */
+static gboolean
+g_local_file_trash (GFile         *file,
+		    GCancellable  *cancellable,
+		    GError       **error)
+{
+  GLocalFile *local = G_LOCAL_FILE (file);
+  SHFILEOPSTRUCTW op = {0};
+  gboolean success;
+  wchar_t *wfilename;
+  long len;
+
+  wfilename = g_utf8_to_utf16 (local->filename, -1, NULL, &len, NULL);
+  /* SHFILEOPSTRUCT.pFrom is double-zero-terminated */
+  wfilename = g_renew (wchar_t, wfilename, len + 2);
+  wfilename[len + 1] = 0;
+
+  op.wFunc = FO_DELETE;
+  op.pFrom = wfilename;
+  op.fFlags = FOF_ALLOWUNDO;
+
+  success = SHFileOperationW (&op) == 0;
+
+  if (success && op.fAnyOperationsAborted)
+    {
+      if (cancellable && !g_cancellable_is_cancelled (cancellable))
+	g_cancellable_cancel (cancellable);
+      g_set_error (error, G_IO_ERROR,
+		   G_IO_ERROR_CANCELLED,
+		   _("Unable to trash file: %s"),
+		   _("Operation was cancelled"));
+      success = FALSE;
+    }
+  else if (!success)
+    g_set_error (error, G_IO_ERROR,
+		 G_IO_ERROR_FAILED,
+		 _("Unable to trash file: %s"),
+		 _("internal error"));
+
+  g_free (wfilename);
+  return success;
+}
+#endif /* G_OS_WIN32 */
 
 static gboolean
 g_local_file_make_directory (GFile         *file,
@@ -1781,8 +1956,7 @@ g_local_file_move (GFile                  *source,
 		   gpointer                progress_callback_data,
 		   GError                **error)
 {
-  GLocalFile *local_source;
-  GLocalFile *local_destination = G_LOCAL_FILE (destination);
+  GLocalFile *local_source, *local_destination;
   struct stat statbuf;
   gboolean destination_exist, source_is_dir;
   char *backup_name;
@@ -1798,14 +1972,17 @@ g_local_file_move (GFile                  *source,
     }
   
   local_source = G_LOCAL_FILE (source);
+  local_destination = G_LOCAL_FILE (destination);
   
   res = g_lstat (local_source->filename, &statbuf);
   if (res == -1)
     {
+      int errsv = errno;
+
       g_set_error (error, G_IO_ERROR,
-		   g_io_error_from_errno (errno),
+		   g_io_error_from_errno (errsv),
 		   _("Error moving file: %s"),
-		   g_strerror (errno));
+		   g_strerror (errsv));
       return FALSE;
     }
   else
@@ -1844,7 +2021,7 @@ g_local_file_move (GFile                  *source,
   if (flags & G_FILE_COPY_BACKUP && destination_exist)
     {
       backup_name = g_strconcat (local_destination->filename, "~", NULL);
-      if (rename (local_destination->filename, backup_name) == -1)
+      if (g_rename (local_destination->filename, backup_name) == -1)
 	{
       	  g_set_error (error,
 		       G_IO_ERROR,
@@ -1861,18 +2038,20 @@ g_local_file_move (GFile                  *source,
     {
       /* Source is a dir, destination exists (and is not a dir, because that would have failed
 	 earlier), and we're overwriting. Manually remove the target so we can do the rename. */
-      res = unlink (local_destination->filename);
+      res = g_unlink (local_destination->filename);
       if (res == -1)
 	{
+          int errsv = errno;
+
 	  g_set_error (error, G_IO_ERROR,
-		       g_io_error_from_errno (errno),
+		       g_io_error_from_errno (errsv),
 		       _("Error removing target file: %s"),
-		       g_strerror (errno));
+		       g_strerror (errsv));
 	  return FALSE;
 	}
     }
   
-  if (rename (local_source->filename, local_destination->filename) == -1)
+  if (g_rename (local_source->filename, local_destination->filename) == -1)
     {
       int errsv = errno;
 
@@ -1937,7 +2116,7 @@ g_local_file_file_iface_init (GFileIface *iface)
   iface->get_uri = g_local_file_get_uri;
   iface->get_parse_name = g_local_file_get_parse_name;
   iface->get_parent = g_local_file_get_parent;
-  iface->contains_file = g_local_file_contains_file;
+  iface->prefix_matches = g_local_file_prefix_matches;
   iface->get_relative_path = g_local_file_get_relative_path;
   iface->resolve_relative_path = g_local_file_resolve_relative_path;
   iface->get_child_for_display_name = g_local_file_get_child_for_display_name;
