@@ -324,11 +324,17 @@ get_mtab_monitor_file (void)
 #endif
 }
 
+#ifndef HAVE_GETMNTENT_R
 G_LOCK_DEFINE_STATIC(getmntent);
+#endif
 
 static GList *
 _g_get_unix_mounts ()
 {
+#ifdef HAVE_GETMNTENT_R
+  struct mntent ent;
+  char buf[1024];
+#endif
   struct mntent *mntent;
   FILE *file;
   char *read_file;
@@ -346,8 +352,12 @@ _g_get_unix_mounts ()
   
   mounts_hash = g_hash_table_new (g_str_hash, g_str_equal);
   
+#ifdef HAVE_GETMNTENT_R
+  while ((mntent = getmntent_r (file, &ent, buf, sizeof (buf))) != NULL)
+#else
   G_LOCK (getmntent);
   while ((mntent = getmntent (file)) != NULL)
+#endif
     {
       /* ignore any mnt_fsname that is repeated and begins with a '/'
        *
@@ -392,7 +402,9 @@ _g_get_unix_mounts ()
   
   endmntent (file);
 
+#ifndef HAVE_GETMNTENT_R
   G_UNLOCK (getmntent);
+#endif
   
   return g_list_reverse (return_list);
 }
@@ -608,6 +620,10 @@ get_fstab_file (void)
 static GList *
 _g_get_unix_mount_points (void)
 {
+#ifdef HAVE_GETMNTENT_R
+  struct mntent ent;
+  char buf[1024];
+#endif
   struct mntent *mntent;
   FILE *file;
   char *read_file;
@@ -622,8 +638,12 @@ _g_get_unix_mount_points (void)
 
   return_list = NULL;
   
+#ifdef HAVE_GETMNTENT_R
+  while ((mntent = getmntent_r (file, &ent, buf, sizeof (buf))) != NULL)
+#else
   G_LOCK (getmntent);
   while ((mntent = getmntent (file)) != NULL)
+#endif
     {
       if ((strcmp (mntent->mnt_dir, "ignore") == 0) ||
 	  (strcmp (mntent->mnt_dir, "swap") == 0))
@@ -661,7 +681,10 @@ _g_get_unix_mount_points (void)
     }
   
   endmntent (file);
+
+#ifndef HAVE_GETMNTENT_R
   G_UNLOCK (getmntent);
+#endif
   
   return g_list_reverse (return_list);
 }
@@ -1843,25 +1866,27 @@ gboolean
 g_unix_mount_guess_should_display (GUnixMountEntry *mount_entry)
 {
   GUnixMountType guessed_type;
+  const char *mount_path;
 
   /* Never display internal mountpoints */
   if (g_unix_mount_is_system_internal (mount_entry))
     return FALSE;
-
-  /* Only display things that look "removable" or
-     things in /media (which are generally user mountable) */
-  guessed_type = g_unix_mount_guess_type (mount_entry);
-  if (guessed_type == G_UNIX_MOUNT_TYPE_IPOD ||
-      guessed_type == G_UNIX_MOUNT_TYPE_CDROM ||
-      guessed_type == G_UNIX_MOUNT_TYPE_FLOPPY ||
-      guessed_type == G_UNIX_MOUNT_TYPE_ZIP ||
-      guessed_type == G_UNIX_MOUNT_TYPE_JAZ ||
-      guessed_type == G_UNIX_MOUNT_TYPE_CAMERA ||
-      guessed_type == G_UNIX_MOUNT_TYPE_MEMSTICK ||
-      (mount_entry->mount_path != NULL &&
-       g_str_has_prefix (mount_entry->mount_path, "/media")))
-    return TRUE;
-
+  
+  /* Only display things in /media (which are generally user mountable)
+     and home dir (fuse stuff) */
+  mount_path = mount_entry->mount_path;
+  if (mount_path != NULL)
+    {
+      if (strstr (mount_path, "/.gvfs") != NULL)
+        return TRUE;
+      
+      if (g_str_has_prefix (mount_path, "/media/"))
+        return TRUE;
+      
+      if (g_str_has_prefix (mount_path, g_get_home_dir ()))
+        return TRUE;
+    }
+  
   return FALSE;
 }
 
@@ -2017,19 +2042,28 @@ _resolve_dev_root (void)
        */
       f = fopen ("/etc/mtab", "r");
       if (f != NULL) {
+	struct mntent *entp;
+#ifdef HAVE_GETMNTENT_R        
         struct mntent ent;
-        
-        while (getmntent_r (f, &ent, buf, sizeof (buf)) != NULL) {
-          
-          if (stat (ent.mnt_fsname, &statbuf) == 0 &&
+        while ((entp = getmntent_r (f, &ent, buf, sizeof (buf))) != NULL) {
+#else
+	G_LOCK (getmntent);
+	while ((entp = getmntent (f)) != NULL) { 
+#endif          
+          if (stat (entp->mnt_fsname, &statbuf) == 0 &&
               statbuf.st_dev == root_dev) {
-            strncpy (real_dev_root, ent.mnt_fsname, sizeof (real_dev_root) - 1);
+            strncpy (real_dev_root, entp->mnt_fsname, sizeof (real_dev_root) - 1);
             real_dev_root[sizeof (real_dev_root) - 1] = '\0';
             fclose (f);
             goto found;
           }
         }
-        fclose (f);
+
+        endmntent (f);
+
+#ifndef HAVE_GETMNTENT_R
+	G_UNLOCK (getmntent);
+#endif
       }                                        
       
       /* no, that didn't work.. next we could scan /dev ... but I digress.. */
