@@ -332,7 +332,7 @@ void                    g_once_init_leave       (volatile gsize *value_location,
 G_INLINE_FUNC gboolean
 g_once_init_enter (volatile gsize *value_location)
 {
-  if G_LIKELY (g_atomic_pointer_get ((void*volatile*) value_location) != 0)
+  if G_LIKELY (g_atomic_pointer_get ((void*volatile*) value_location) != NULL)
     return FALSE;
   else
     return g_once_init_enter_impl (value_location);
